'use strict';

/**
 * Gestionnaire d'envoi de fichiers.
 *
 * @license http://www.gnu.org/licenses/gpl.html
 * @link http://www.igalerie.org/
 */
function Upload(prefix, options)
{
	// Options.
	this.options;



	// Index du fichier courant.
	var fileIndex = 0;

	// Code HTML se trouvant dans la liste des fichiers.
	var innerList = $('#' + prefix + 'list').html();

	// Liste des fichiers ajoutés.
	var list = [];

	// Indique si au moins un fichier a été envoyé avec succès.
	var success = false;

	// Indique si on a commencé l'upload.
	var start = false;

	// This!
	var This = this;

	// Total des fichiers envoyés.
	var totalUploadFiles;



	// Options.
	This.options = options;

	// Bouton "Sélectionner des fichiers".
	$('#' + prefix + 'input_file').hide().on('change', function(f)
	{
		addFiles(this.files);
	});
	$(document).on('click', '#' + prefix + 'add', function()
	{
		if (!start)
		{
			$('#' + prefix + 'input_file').click();
		}
		return false;
	});

	// Bouton "Vider la liste".
	$(document).on('click', '#' + prefix + 'clear', function()
	{
		if ($(this).prop('disabled'))
		{
			return false;
		}
		$(this).prop('disabled', true);

		// On réinitialise tout.
		fileIndex = 0;
		list = [];

		// On supprime la liste de fichiers.
		$('.' + prefix + 'file').remove();

		// On remet le contenu de démarrage.
		$(innerList).appendTo('#' + prefix + 'list');

		// Mise à jour du nombre de fichiers.
		updateInterface();

		return false;
	});

	// Bouton "Envoyer".
	$(document).on('click', '#' + prefix + 'start', function()
	{
		if ($(this).prop('disabled') || start || !list.length)
		{
			return false;
		}

		// Si aucun album sélectionné, on ne va pas plus loin.
		if (!This.options.ajaxData.album_id)
		{
			alert(This.options.text.noAlbum);
			return false;
		}

		// On indique que l'on vient de démarrer l'envoi,
		// et on désactive les boutons d'ajout et d'envoi.
		start = true;
		$('#' + prefix + 'add,#' + prefix + 'start').prop('disabled', true);

		// On scroll en haut de la liste de fichiers
		// avant de démarrer l'envoi des fichiers.
		$('#' + prefix + 'list').animate({ scrollTop: 0 }, 250, 'swing', function()
		{
			setTimeout(function()
			{
				fileIndex = 0;
				totalUploadFiles = getTotalFiles();
				totalUploadFiles.loadedFiles = 0;
				totalUploadFiles.loadedSize = 0;
				uploadFile();
			}, 250);
		});

		return false;
	});

	// Suppression d'un fichier.
	$(document).on('click', '.' + prefix + 'file_delete', function()
	{
		if (!$(this).parents('.' + prefix + 'file').hasClass(prefix + 'file_warning'))
		{
			var i = $(this).parents('.' + prefix + 'file').attr('id')
				.replace(prefix + 'file_', '');
			list[i] = false;
		}

		// On supprime le code HTML du fichier.
		$(this).parents('.' + prefix + 'file').animate(
		{
			opacity: 0,
			height: 0
		},
		{
			duration: 300,
			easing: 'swing',
			complete: function()
			{
				$(this).remove();
				if (!start && $('.' + prefix + 'file').length == 0)
				{
					$(innerList).appendTo('#' + prefix + 'list');
				}

				// Mise à jour du nombre de fichiers.
				updateInterface();
			}
		});

		return false;
	});

	// Drag & drop.
	$(document).on('dragover', '#' + prefix + 'list', function()
	{
		if (!start)
		{
			$(this).removeClass().addClass('dragover');
		}
		return false;
	});
	$(document).on('dragleave', '#' + prefix + 'list', function()
	{
		if (!start)
		{
			$(this).removeClass().addClass('dragleave');
		}
		return false;
	});
	$(document).on('drop', '#' + prefix + 'list', function(e)
	{
		if (!start
		&& e.originalEvent.dataTransfer
		&& e.originalEvent.dataTransfer.files.length)
		{
			$(this).removeClass().addClass('drop');
			addFiles(e.originalEvent.dataTransfer.files);
		}
		else
		{
			$(this).removeClass();
		}
		return false;
	});

	updateInterface();



	/**
	 * Ajout de nouveaux fichiers.
	 *
	 * @param object files
	 * @return void
	 */
	function addFiles(files)
	{
		if (start)
		{
			return;
		}

		// On supprime tout le code HTML présent dans la liste des fichiers.
		$('#' + prefix + 'list > :not(.' + prefix + 'file)').remove();

		// On désactive les boutons.
		$('#' + prefix + 'add,#' + prefix + 'clear,#' + prefix + 'start').prop('disabled', true);

		// Vérification des fichiers.
		checkFile(files, 0);
	};

	/**
	 * Ajoute un fichier à la liste.
	 *
	 * @param object files
	 * @param int i
	 * @param boolean check
	 * @return void
	 */
	function addToList(files, i, check)
	{
		// Code HTML du fichier.
		var html = {};
		if (check === true)
		{
			html.progress = '<div><div></div></div>';
			html.id = '<div id="' + prefix + 'file_' + fileIndex
				+ '" class="' + prefix + 'file">';
		}
		else
		{
			html.progress = check;
			html.id = '<div class="' + prefix + 'file ' + prefix + 'file_warning">';
		}
		var html = html.id + '<div class="' + prefix + 'file_body"><a class="' + prefix
			+ 'file_delete" href="javascript:;"></a><div class="' + prefix
			+ 'file_infos"><span class="' + prefix + 'file_name">'
			+ formatFilename(files[i].name) + '</span><span class="' + prefix
			+ 'file_size">' + formatFilesize(files[i].size) + '</span></div><div class="'
			+ prefix + 'file_progress">' + html.progress + '</div></div></div>';
		$(html).appendTo('#' + prefix + 'list');

		// On ajoute le fichier à la liste.
		if (check == true)
		{
			list[fileIndex] = files[i];
			fileIndex++;
		}

		// On scroll en bas de la liste.
		$('#' + prefix + 'list').scrollTop($('#' + prefix + 'list')[0].scrollHeight);

		// Mise à jour du nombre de fichiers.
		updateInterface(true);

		if (typeof files[i + 1] == 'undefined')
		{
			// On réactive les boutons.
			$('#' + prefix + 'add,#' + prefix + 'clear').prop('disabled', false);
			if (This.options.albumSelected && list.length)
			{
				$('#' + prefix + 'start').prop('disabled', false);
			}
		}
		else
		{
			checkFile(files, i + 1);
		}
	};

	/**
	 * Vérifie un fichier.
	 *
	 * @parram object files
	 * @parram int i
	 * @return boolean
	 */
	function checkFile(files, i)
	{
		var file = files[i];
		var total = getTotalFiles();

		// Nombre de fichiers.
		if (total.files == This.options.maxTotalFiles)
		{
			addToList(files, i, getFileError('totalfiles'));
			return;
		}

		// Nom de fichier.
		var regexp = new RegExp('^.{1,250}\.(' + This.options.fileExts.join('|') + ')$', 'i');
		if (!file.name.match(regexp))
		{
			addToList(files, i, getFileError('filename'));
			return;
		}

		// Noms de fichiers identiques.
		for (var n = 0; n < list.length; n++)
		{
			if (list[n] && list[n].name == file.name)
			{
				addToList(files, i, getFileError('sameFilename'));
				return;
			}
		}

		// Type de fichier.
		if ($.inArray(file.type, This.options.fileTypes) == -1)
		{
			addToList(files, i, getFileError('filetype'));
			return;
		}

		// Poids du fichier.
		if (file.size > This.options.maxFileSize)
		{
			addToList(files, i, getFileError('filesize'));
			return;
		}

		// Poids total.
		if ((total.size + file.size) > This.options.maxTotalSize)
		{
			addToList(files, i, getFileError('totalsize'));
			return;
		}

		// Dimensions de l'image.
		if (file.type.match(/^image/))
		{
			var f = new FileReader;
			f.onload = function()
			{
				var img = new Image;
				img.onload = function()
				{
					addToList(files, i, (img.width > This.options.maxWidth
						|| img.height > This.options.maxHeight)
						? getFileError('imagesize')
						: true);
				};
				img.src = f.result;
			};
			f.readAsDataURL(file);
		}
		else
		{
			addToList(files, i, true);
		}
	};

	/**
	 * Formate le nom de fichier.
	 *
	 * @param string str
	 * @return string
	 */
	function formatFilename(str)
	{
		// On limite la longueur du nom de fichier.
		var max = This.options.maxFileNameLength;
		if (max)
		{
			max -= 9;
			if (str.length > max)
			{
				var regexp = new RegExp('^(.{' + max + '}).+(\..{3,4})$');
				str = str.replace(regexp, '$1[...]$2')
			}
		}

		return str;
	};

	/**
	 * Formate le poids d'un fichier.
	 *
	 * @param integer filesize
	 * @param string unit
	 * @return string
	 */
	function formatFilesize(filesize, unit)
	{
		var kb = 1024,
			mb = 1024 * kb,
			gb = 1024 * mb;

		// Forcer l'utilisation d'une unité ?
		if (unit)
		{
			unit = (unit == 'kb') ? kb : mb;
			filesize = String(Math.round(filesize/unit * 100) / 100);
		}
		else
		{
			if (filesize < kb)
			{
				filesize = This.options.text.sizeUnits[0].replace(
					'%s', filesize
				);
			}
			else if (filesize < mb)
			{
				filesize = This.options.text.sizeUnits[1].replace(
					'%s', Math.round(filesize/kb * 100) / 100
				);
			}
			else if (filesize < gb)
			{
				filesize = This.options.text.sizeUnits[2].replace(
					'%s', Math.round(filesize/mb * 100) / 100
				);
			}
			else
			{
				filesize = This.options.text.sizeUnits[3].replace(
					'%s', Math.round(filesize/gb * 100) / 100
				);
			}
		}

		return filesize.replace('.', This.options.text.decimalSeparator);
	};

	/**
	 * Retourne le code HTML d'un message d'erreur.
	 *
	 * @param string type
	 * @return string
	 */
	function getFileError(type)
	{
		return '<p class="' + prefix + 'warning">' + This.options.text.warning[type] + '</p>';
	};

	/**
	 * Retourne le nombre et le poids total des fichiers ajoutés à la liste.
	 *
	 * @return object
	 */
	function getTotalFiles()
	{
		for (var i = 0, n = 0, size = 0; i < list.length; i++)
		{
			if (list[i])
			{
				n++;
				size += list[i].size;
			}
		}
		return { files: n, size: size };
	};

	/**
	 * Met à jour divers éléments d'interface
	 * lorsqu'on ajoute ou supprime des fichiers.
	 *
	 * @param boolean only_stats
	 * @return void
	 */
	function updateInterface(only_stats)
	{
		// Informations sur la liste.
		var total = getTotalFiles();
		$('#' + prefix + 'infos_files').text(
			total.files + '/' + This.options.text.files.replace('%s', This.options.maxTotalFiles)
		);
		$('#' + prefix + 'infos_filesize').text(
			formatFilesize(total.size) + '/' + formatFilesize(This.options.maxTotalSize)
		);

		if (start || only_stats)
		{
			return;
		}

		// Boutons.
		if (!total.files)
		{
			$('#' + prefix + 'start').prop('disabled', true);
		}

		if ($('.' + prefix + 'file').length)
		{
			$('#' + prefix + 'clear').prop('disabled', false);
		}
		else
		{
			$('#' + prefix + 'clear').prop('disabled', true);
		}
	};

	/**
	 * Envoi un fichier.
	 *
	 * @return void
	 */
	function uploadFile()
	{
		var f = list[fileIndex];

		// S'il n'y a pas de fichier.
		if (!f)
		{
			// Si le fichier a été supprimé, on passe au suivant.
			if (f === false)
			{
				fileIndex++;
				uploadFile();
			}

			// Fin de l'upload.
			else if (success)
			{
				$('#' + prefix + 'form').submit();
			}
			else
			{
				start = false;
				$('#' + prefix + 'add,#' + prefix + 'start').prop('disabled', false);
			}

			return;
		}

		// Lecture du fichier.
		var reader = new FileReader();
		reader.readAsDataURL(f);
		reader.onerror = function(e)
		{
			console.log('File could not be read (' + e.target.error.code + ')');
		};
		reader.onload = function(e)
		{
			// On envoi le fichier en parties de longueur maximum slice_size.
			var filedata = e.target.result.split(',')[1];
			var slice_size = This.options.sliceSize;
			var slice_max = Math.max(Math.ceil(filedata.length / slice_size), 1);
			var error = false;
			var filesize = filedata.length;

			send_part(1);

			function send_part(slice_num)
			{
				This.options.ajaxData.filename = encodeURI(f.name);
				This.options.ajaxData.filedata = filedata.slice(0, slice_size);
				This.options.ajaxData.last_part = (slice_num == slice_max) ? 1 : 0;

				if (slice_num < slice_max)
				{
					filedata = filedata.slice(slice_size);
				}

				$.ajax(
				{
					type: 'POST',
					url: This.options.ajaxScript,
					data: This.options.ajaxData,
					dataType: 'json',
					xhr: function()
					{
						var xhr = new XMLHttpRequest();
						xhr.upload.onprogress = function(e)
						{
							if (e.lengthComputable)
							{
								// Pourcentage du fichier envoyé.
								var avg = filesize / slice_max;
								var pc = (e.loaded * 100) / e.total;
								var filesize_loaded = (avg * (pc / 100)) + ((slice_num - 1) * avg);
								var filesize_pc = (filesize_loaded * 100) / filesize;
								if (filesize_pc > 100)
								{
									filesize_pc = 100;
								}

								$('#' + prefix + 'file_' + fileIndex
								+ ' .' + prefix + 'file_progress div div')
									.css({width: Math.round(filesize_pc) + '%'});

								// Pourcentage de tous les fichiers envoyés.
								var loaded_size = totalUploadFiles.loadedSize
									+ (f.size * (filesize_pc / 100));
								var total_pc = Math.round((loaded_size * 100)
									/ totalUploadFiles.size);

								$('#' + prefix + 'infos_progress_pc').text(total_pc + '%');
							}
						};
						return xhr;
					},

					// Envoi terminé.
					complete: function(r)
					{
						// S'il n'y a pas eu d'erreur.
						if (!error)
						{
							// S'il reste des parties de fichiers à envoyer, on continue.
							if (slice_num < slice_max)
							{
								send_part(slice_num + 1);
								return;
							}

							// Sinon, on ajoute le nom de fichier au formulaire.
							if (typeof r.responseJSON == 'object')
							{
								var filename = r.responseJSON.filename;
								var message = r.responseJSON.message;
								message = (message != '' && message != 'OK.')
									? (filename ? encodeURI(filename) + ' : ' : '')
										+ encodeURI(message)
									: encodeURI(filename);
								$('#' + prefix + 'list').after(
									'<input name="' + r.responseJSON.status +
									'[]" type="hidden" value="' + message + '">'
								);
							}
						}

						// On passe au fichier suivant.
						totalUploadFiles.loadedSize += f.size;
						fileIndex++;
						uploadFile();
					},

					// Une erreur ?
					error: function(r)
					{
						$('#' + prefix + 'file_' + fileIndex).addClass(prefix + 'file_error');
						$('#' + prefix + 'file_' + fileIndex + ' .' + prefix + 'file_progress')
							.html('<p>' + This.options.text.failed + '</p>');
						error = true;
						console.log(r.responseText);
					},

					// Fichier envoyé avec succès.
					success: function(r)
					{
						if (typeof r == 'object')
						{
							console.log(r.status + ': ' + r.message);
						}

						if (slice_num < slice_max)
						{
							return;
						}

						var file_index = fileIndex;

						// On indique que l'envoi a été un succès.
						$('#' + prefix + 'file_' + fileIndex)
							.addClass(prefix + 'file_success');
						$('#' + prefix + 'file_' + fileIndex
							+ ' .' + prefix + 'file_progress').html(
							'<p>' + This.options.text.success + '</p>');

						// Puis on retire le fichier de la liste.
						list[file_index] = false;
						updateInterface();

						setTimeout(function()
						{
							$('#' + prefix + 'file_' + file_index
								+ ' .' + prefix + 'file_delete').click();
						}, 2000);

						// On indique qu'au moins un fichier a été envoyé avec succès.
						success = true;
					}
				});
			}
		};
	};
};